#include "SnailShellExt.h"
#include "Snail.h"
#include "kmk_parser.h"
#include "stringutil.h"
#include "fileutil.h"
#include "trace.h"
#include "resource.h"

ULONG SnailShellExt::_global_ref_counter = 0L;

SnailShellExt::SnailShellExt(void) : _ref_counter(0), m_pDataObj(NULL)
{
    trace("hInst = 0x%x\n", hInst);
    InterlockedIncrement((PLONG) &_global_ref_counter);

    char buffer[1024];

    LoadString(hInst, IDS_COMPILE, buffer, 1024);
    _compile = buffer;

    LoadString(hInst, IDS_COMPILE_DESC, buffer, 1024);
    _compile_desc = buffer;

    LoadString(hInst, IDS_CLEAN, buffer, 1024);
    _clean = buffer;

    LoadString(hInst, IDS_CLEAN_DESC, buffer, 1024);
    _clean_desc = buffer;

    GetModuleFileName(hInst, buffer, 1024);
    _self_path = euc::getpathname(buffer);
}

SnailShellExt::~SnailShellExt(void)
{
    InterlockedDecrement((PLONG) &_global_ref_counter);
}

// IUnknown members
STDMETHODIMP SnailShellExt::QueryInterface(REFIID iid, LPVOID FAR *object)
{
    *object = NULL;

    if (IsEqualIID(iid, IID_IUnknown))
        *object = (LPUNKNOWN)(LPSHELLEXTINIT) this;
    else if (IsEqualIID(iid, IID_IShellExtInit))
        *object = (LPSHELLEXTINIT) this;
    else if (IsEqualIID(iid, IID_IContextMenu))
        *object = (LPCONTEXTMENU) this;

    if (*object)
    {
        AddRef();
        return NOERROR;
    }

    return E_NOINTERFACE;
}

STDMETHODIMP_(ULONG) SnailShellExt::AddRef(void)
{
    return InterlockedIncrement((LPLONG)& _ref_counter);
}

STDMETHODIMP_(ULONG) SnailShellExt::Release(void)
{
    if (InterlockedDecrement((LPLONG)& _ref_counter))
        return _ref_counter;
    delete this;
    return 0L;
}

// IShell members
STDMETHODIMP SnailShellExt::QueryContextMenu(HMENU hMenu, UINT indexMenu,
    UINT idCmdFirst, UINT idCmdLast, UINT uFlags)
{
    _command.clear();

    FORMATETC fmte = { CF_HDROP,
              (DVTARGETDEVICE FAR *) NULL,
              DVASPECT_CONTENT,
              -1,
              TYMED_HGLOBAL };
    STGMEDIUM medium;
    UINT cbFiles = 0;

    m_pDataObj->GetData(&fmte, &medium);
    if (medium.hGlobal)
        cbFiles = DragQueryFile((HDROP) medium.hGlobal, (UINT) -1, 0, 0);

    for (UINT i = 0; i < cbFiles; ++i)
    {
        UINT nFileLength = DragQueryFile((HDROP) medium.hGlobal, i, NULL, 0);
        char *pszFileName = new char[nFileLength + 1];

        DragQueryFile((HDROP) medium.hGlobal, i, pszFileName,
            nFileLength + 1);
        _filename = pszFileName;

        delete[] pszFileName;
    }

    if (cbFiles > 1)
        return 0;

    std::list<std::string> configurations = parse_kmk(_filename);
    UINT count = 0;

    if (configurations.size() == 0)
        return 0;

    HMENU hSub = CreatePopupMenu();
    InsertMenu(hMenu, indexMenu++, MF_STRING | MF_BYPOSITION | MF_POPUP,
        (UINT_PTR) hSub, _compile.c_str());

    for (std::list<std::string>::iterator ite = configurations.begin();
        ite != configurations.end(); ++ite)
    {
        AppendMenu(hSub, MF_STRING, idCmdFirst + count, ite->c_str());
        _command[count] = *ite;
        ++count;
    }

    _config_count = count;

    hSub = CreatePopupMenu();
    InsertMenu(hMenu, indexMenu++, MF_STRING | MF_BYPOSITION | MF_POPUP,
        (UINT_PTR) hSub, _clean.c_str());

    for (std::list<std::string>::iterator ite = configurations.begin();
        ite != configurations.end(); ++ite)
    {
        AppendMenu(hSub, MF_STRING, idCmdFirst + count, ite->c_str());
        ++count;
    }

    return count;
}

STDMETHODIMP SnailShellExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpcmi)
{
    HRESULT hr = E_INVALIDARG;

    // If HIWORD(lpcmi->lpVerb) then we have been called programmatically
    // and lpVerb is a command that should be invoked.  Otherwise, the shell
    // has called us, and LOWORD(lpcmi->lpVerb) is the menu ID the user has
    // selected.  Actually, it's (menu ID - idCmdFirst) from QueryContextMenu().
    if (!HIWORD(lpcmi->lpVerb))
    {
        UINT idCmd = LOWORD(lpcmi->lpVerb);

        UINT idCmdIdx = idCmd % _config_count;

        std::string cmdline = _self_path + "kmake.bat";
        std::string param = "\"" + _filename + "\" CFG=" + _command[idCmdIdx];

        if (idCmd >= _config_count)
            param += " clean";

        ShellExecute(NULL, NULL, cmdline.c_str(), param.c_str(), NULL, SW_SHOW);
        hr = NOERROR;
    }

    return hr;
}

STDMETHODIMP SnailShellExt::GetCommandString(UINT idCmd, UINT uFlags,
    UINT FAR *reserved, LPSTR pszName, UINT cchMax)
{
    std::string _desc = (idCmd >= _config_count)
        ? _clean_desc : _compile_desc;
    UINT idCmdIdx = idCmd % _config_count;

    std::string command = euc::replace_all(_desc, "%1",
        _command[idCmdIdx]);
    if (uFlags == GCS_HELPTEXT && cchMax > command.size())
        strcpy(pszName, command.c_str());

    return NOERROR;
}

// IShellExtInit methods
STDMETHODIMP SnailShellExt::Initialize(LPCITEMIDLIST pIDFolder,
    LPDATAOBJECT pDataObj, HKEY hKeyID)
{
    // Initialize can be called more than once
    if (m_pDataObj)
        m_pDataObj->Release();

    // duplicate the object pointer and registry handle

    if (pDataObj)
    {
        m_pDataObj = pDataObj;
        pDataObj->AddRef();
    }

    return NOERROR;
}

bool SnailShellExt::CanUnloadNow(void)
{
    return _global_ref_counter == 0;
}
